/*
    Copyright (C) 2001-2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <math.h>

#include <kdebug.h>

#include "SVGColorImpl.h"
#include "SVGDocumentImpl.h"
#include "SVGStylableImpl.h"
#include "SVGAnimateColorElementImpl.h"

using namespace KSVG;

SVGAnimateColorElementImpl::SVGAnimateColorElementImpl(DOM::ElementImpl *impl) : SVGAnimationElementImpl(impl)
{
	m_firstEvent = true;

	m_fromColor = new SVGColorImpl(this);
	m_fromColor->ref();

	m_toColor = new SVGColorImpl(this);
	m_toColor->ref();
}

SVGAnimateColorElementImpl::~SVGAnimateColorElementImpl()
{
	m_fromColor->deref();
	m_toColor->deref();
}

/*
 * Outstanding issues
 * - 'values' support
 * - 'by' support
 */

void SVGAnimateColorElementImpl::setAttributes()
{
	SVGAnimationElementImpl::setAttributes();
	
	SVGStylableImpl::setColor(getFrom(), m_fromColor);
	SVGStylableImpl::setColor(getTo(), m_toColor);

	ownerDoc()->timeScheduler()->addTimer(this, int(getStartTime() * 1000.0), true);
}

void SVGAnimateColorElementImpl::handleTimerEvent()
{
	QColor fromColor(m_fromColor->rgbColor().color());
	QColor toColor(m_toColor->rgbColor().color());

	if(m_firstEvent)
	{
		m_firstEvent = false;
		m_step = 0;
		m_steps = (int) (getSimpleDuration() * 1000.0) / interval();
	
		m_red = fromColor.red();
	  	m_green = fromColor.green();
		m_blue = fromColor.blue();
	}
	else
	{
		m_step = (int) rint(static_cast<double>(getSimpleDuration() * 1000.0) / static_cast<double>(interval()) * (m_step / static_cast<double>(m_steps)));
		m_steps = (int) (getSimpleDuration() * 1000.0) / interval();
		
		m_red = ((toColor.red() - fromColor.red()) / static_cast<double>(m_steps)) * m_step + fromColor.red();
		m_green = ((toColor.green() - fromColor.green()) / static_cast<double>(m_steps)) * m_step + fromColor.green();
		m_blue =((toColor.blue() - fromColor.blue()) / static_cast<double>(m_steps)) * m_step + fromColor.blue();

		QString color = "rgb(" + QString::number((int) m_red) + "," + QString::number((int) m_green) + "," + QString::number((int) m_blue) + ")";
		applyAttribute(getAttributeName(), color);
	}

	if(m_step < m_steps)
	{
		ownerDoc()->timeScheduler()->addTimer(this, interval(), true);
		m_step++;
	}
}

// vim:ts=4:noet
