/*
    Copyright (C) 2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "SVGDocumentImpl.h"
#include "KSVGCanvas.h"
#include "SVGTimeScheduler.moc"

using namespace KSVG;

SVGTimeScheduler::SVGTimeScheduler(SVGDocumentImpl *doc) : QObject(), m_doc(doc)
{
	m_directStart = false;
}

SVGTimeScheduler::~SVGTimeScheduler()
{
    SVGTimerList::iterator it;
    for(it = m_timerList.begin(); it != m_timerList.end(); ++it)
	{
		SVGTimer *svgTimer = *it;
		delete svgTimer;
	}
}

void SVGTimeScheduler::addTimer(SVGElementImpl *element, unsigned int ms, bool singleShot)
{
	// Check if there's another timer which fits the same requirements
	// If yes, append ourselves to the notify element list of that timer
    SVGTimerList::iterator it;
    for(it = m_timerList.begin(); it != m_timerList.end(); ++it)
	{
		SVGTimer *svgTimer = *it;
		
		// Only "attach" to it, if it wasn't invoked yet...
		if(svgTimer && svgTimer->fits(element, ms, singleShot))
			return;
	}

	// SVGTimer takes care of registering to the notification list...
	SVGTimer *svgTimer = new SVGTimer(this, element, ms, singleShot);
	m_timerList.append(svgTimer);

	if(m_directStart)
		startTimers();
}

void SVGTimeScheduler::startTimers()
{
	if(!m_directStart)
		m_directStart = true;

    SVGTimerList::iterator it;
    for(it = m_timerList.begin(); it != m_timerList.end(); ++it)
	{
		SVGTimer *svgTimer = *it;
		if(svgTimer && !svgTimer->invoked())
		{
			svgTimer->setInvoked(true);
			svgTimer->start(this, SLOT(slotTimerNotify()));
		}
	}
}
	
void SVGTimeScheduler::stopTimers()
{
    SVGTimerList::iterator it;
    for(it = m_timerList.begin(); it != m_timerList.end(); ++it)
	{
		SVGTimer *svgTimer = *it;
		if(svgTimer)
		{
			svgTimer->setInvoked(false);
			svgTimer->stop();
		}
	}
}

void SVGTimeScheduler::slotTimerNotify()
{
	QTimer *senderTimer = const_cast<QTimer *>(static_cast<const QTimer *>(sender()));

	SVGTimer *svgTimer = 0;
	SVGTimerList::iterator it;
    for(it = m_timerList.begin(); it != m_timerList.end(); ++it)
	{
		SVGTimer *cur = *it;
		if(*cur == senderTimer)
		{
			svgTimer = cur;
			break;
		}
	}

	if(!svgTimer)
		return;

	svgTimer->notifyAll();

	QTime t;
	t.start();
	
	// Animations need direct updates
	m_doc->canvas()->update();
	emit m_doc->finishedRendering();

	svgTimer->changeInterval(t.elapsed());

	if(svgTimer->singleShot())
		m_timerList.remove(svgTimer);
}

// vim:ts=4:noet
