/*
    Copyright (C) 2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef SVGTimeScheduler_H
#define SVGTimeScheduler_H

#include <qtimer.h>
#include <qobject.h>
#include <qvaluelist.h>

#include "SVGElementImpl.h"
#include "SVGAnimationElementImpl.h"

namespace KSVG
{

typedef QValueList<SVGElementImpl *> SVGNotifyList;
class SVGTimer
{
public:
    SVGTimer() : m_ms(0), m_timer(0)
    {
		m_invoked = false;
		m_singleShot = false;
	}

	SVGTimer(QObject *scheduler, SVGElementImpl *element, unsigned int ms, bool singleShot)
	{
		m_ms = ms;
		m_invoked = false;
		m_singleShot = singleShot;
		m_timer = new QTimer(scheduler);
		m_notifyList.append(element);
	}
	
    ~SVGTimer()
	{
		delete m_timer;
	}

	bool operator==(const QTimer *timer) { return (m_timer == timer); }

	void start(QObject *receiver, const char *member)
	{
		QObject::connect(m_timer, SIGNAL(timeout()), receiver, member);
		m_timer->start(m_ms, m_singleShot);
	}
	
	void stop()
	{
		m_timer->disconnect(SIGNAL(timeout()));
		m_timer->stop();
	}

	unsigned int ms() const { return m_ms; }
	void setMs(unsigned int ms) { m_ms = ms; }
	
	bool invoked() const { return m_invoked; }
	void setInvoked(bool invoked) { m_invoked = invoked; }
	
	bool singleShot() const { return m_singleShot; }
	void setSingleShot(bool singleShot) { m_singleShot = singleShot; }

	unsigned int notifyListCount() { return m_notifyList.count(); }

	bool fits(SVGElementImpl *element, unsigned int ms, bool singleShot)
	{
		if(m_ms == ms && m_singleShot == singleShot && !m_invoked)
		{
			m_notifyList.append(element);
			return true;
		}

		return false;
	}

	void notifyAll()
	{
		SVGNotifyList::iterator it;
		for(it = m_notifyList.begin(); it != m_notifyList.end(); ++it)
		{
			SVGElementImpl *element = *it;
			if(!element)
				continue;

			SVGAnimationElementImpl *animation = dynamic_cast<SVGAnimationElementImpl *>(element);
			if(animation)
				animation->handleTimerEvent();
		}
	}

	void removeNotify(SVGElementImpl *element)
	{
		m_notifyList.remove(element);
	}

	void changeInterval(unsigned int ms)
	{
		SVGNotifyList::iterator it;
		for(it = m_notifyList.begin(); it != m_notifyList.end(); ++it)
		{
			SVGElementImpl *element = *it;
			if(!element)
				continue;

			SVGAnimationElementImpl *animation = dynamic_cast<SVGAnimationElementImpl *>(element);
			if(animation)
				animation->setInterval(ms);
		}
	}

private:
	unsigned int m_ms;
	bool m_invoked, m_singleShot;

	QTimer *m_timer;
    SVGNotifyList m_notifyList;
};

typedef QValueList<SVGTimer *> SVGTimerList;
class SVGDocumentImpl;
class SVGTimeScheduler : public QObject
{
Q_OBJECT
public:
	SVGTimeScheduler(SVGDocumentImpl *doc);
	~SVGTimeScheduler();
	
	void addTimer(SVGElementImpl *element, unsigned int ms, bool singleShot);
	void startTimers();
	void stopTimers();

private slots:
	void slotTimerNotify();

private:
	SVGDocumentImpl *m_doc;
	SVGTimerList m_timerList;
	bool m_directStart;
};

};

#endif

// vim:ts=4:noet
