// Very small test app to look at ksvg performance without konqueror overhead (Rob)

#include "SVGTestWidget.moc"
#include "SVGDocumentImpl.h"
#include "SVGElementImpl.h"
#include "CanvasFactory.h"
#include "KSVGCanvas.h"
#include <kcursor.h>
#include "SVGSVGElementImpl.h"
#include <iostream>

using namespace KSVG;

SVGTestWidget::SVGTestWidget(const KURL &url)
{
	setBackgroundMode(NoBackground);

	m_doc = new SVGDocumentImpl();
	m_doc->ref();
	resize(450, 450);
	m_canvas = CanvasFactory::request()->loadCanvas(450, 450);
	if(!m_canvas)
		return;
	m_canvas->setup(this, this);
	m_doc->attach(m_canvas);
	m_doc->addToDocumentDict(m_doc->handle(), m_doc);
	connect(m_doc, SIGNAL(gotDimension(float, float)), this, SLOT(slotGotDimension(float, float)));
	connect(m_doc, SIGNAL(finishedRendering()), this, SLOT(slotRenderingFinished()));
	m_doc->open(url);
}

SVGTestWidget::~SVGTestWidget()
{
	m_doc->deref();
	delete m_canvas;
}

void SVGTestWidget::slotGotDimension(float w, float h)
{
	m_canvas->setRenderBufferSize(int(ceil(w)), int(ceil(h)));
	resize(int(ceil(w)), int(ceil(h)));
}

void SVGTestWidget::slotRenderingFinished()
{
	QRect rect(0, 0, width(), height());
	m_canvas->blit(rect, true);
}

void SVGTestWidget::paintEvent(QPaintEvent *e)
{
	m_doc->slotPaint();
	m_canvas->blit(e->rect(), true);
}

void SVGTestWidget::resizeEvent(QResizeEvent *e)
{
	int w = e->size().width();
	int h = e->size().height();

	m_canvas->resize(w, h);
	resize(w, h);
	m_canvas->blit();
}

KSVG::SVGMouseEventImpl *SVGTestWidget::newMouseEvent(KSVG::SVGEventImpl::EventId id, QMouseEvent *event)
{
	DOM::AbstractView temp;

	int clientX = event->x();
	int clientY = event->y();

	if(m_doc && m_doc->rootElement())
	{
		clientX = int(clientX / m_doc->rootElement()->currentScale());
		clientY = int(clientY / m_doc->rootElement()->currentScale());
	}

	int button = 0;
	if(event->stateAfter() & Qt::LeftButton)
		button = 1;
	else if(event->stateAfter() & Qt::MidButton)
		button = 2;
	else if(event->stateAfter() & Qt::RightButton)
		button = 3;

	KSVG::SVGMouseEventImpl *mev = new KSVG::SVGMouseEventImpl(id, // type
			true, // canBubbleArg
			true, // cancelableArg
			temp, // view
			0, // detail
			event->globalX(), //screenXArg
			event->globalY(), // screenYArg,
			clientX, // clientXArg
			clientY, // clientYArg
			(event->state() & Qt::ControlButton), // ctrlKeyArg
			(event->state() & Qt::AltButton), // altKeyArg
			(event->state() & Qt::ShiftButton), // shiftKeyArg
			(event->state() & Qt::MetaButton), // metaKeyArg
			button, // buttonArg
			0);

	mev->ref();

	return mev;
}

void SVGTestWidget::mousePressEvent(QMouseEvent *event)
{
	KSVG::SVGMouseEventImpl *mev = newMouseEvent(KSVG::SVGEventImpl::MOUSEDOWN_EVENT, event);

	if(m_doc && m_doc->rootElement())
		m_doc->rootElement()->prepareMouseEvent(event->pos(), event->pos(), mev);
	mev->deref();
}

void SVGTestWidget::mouseReleaseEvent(QMouseEvent *event)
{
	// only simulate mouse clicks for now
	KSVG::SVGMouseEventImpl *mev = newMouseEvent(KSVG::SVGEventImpl::MOUSEUP_EVENT, event);

	if(m_doc && m_doc->rootElement())
		m_doc->rootElement()->prepareMouseEvent(event->pos(), event->pos(), mev);
	mev->deref();
}

// vim:ts=4:noet
